@@ -22,11 +22,9 @@
                         var events = attr.nodeValue;
                         var ajax = $(this);
                         ajax.unbind(events);
-                        if (ajax.attr('ajax:action')) {
-                            ajax.bind(events, bdajax._action_handler);
-                        }
-                        if (ajax.attr('ajax:event')) {
-                            ajax.bind(events, bdajax._event_handler);
+                        if (ajax.attr('ajax:action')
+                         || ajax.attr('ajax:event')) {
+                            ajax.bind(events, bdajax._dispatching_handler);
                         }
                     }
                 }
@@ -86,6 +84,9 @@
             if (idx != -1) {
                 url = url.substring(0, idx);
             }
+            if (url.charAt(url.length - 1) == '/') {
+                url = url.substring(0, url.length - 1);
+            }
             return url;
         },
         
@@ -129,8 +130,9 @@
                     if (parseInt(status, 10) === 403) {
                         window.location.pathname = bdajax.default_403;
                     } else {
-                        bdajax.error(
-                            '<strong>' + status + '</strong> ' + exception);
+                        var message = '<strong>' + status + '</strong> ';
+                        message += exception;
+                        bdajax.error(message);
                     }
                 };
             }
@@ -169,45 +171,75 @@
                         bdajax.error('Empty response');
                         bdajax.spinner.hide();
                     } else {
-                        var mode = data.mode;
-                        var selector = data.selector;
-                        if (mode == 'replace') {
-                            $(selector).replaceWith(data.payload);
-                            var context = $(selector);
-                            if (context.length) {
-                                context.parent().bdajax();
-                            } else {
-                                $(document).bdajax();
-                            }
-                        } else if (mode == 'inner') {
-                            $(selector).html(data.payload);
-                            $(selector).bdajax();
-                        }
+                        bdajax.fiddle(data.payload, data.selector, data.mode);
                         bdajax.continuation(data.continuation);
                     }
                 }
             });
         },
         
-        continuation: function(actions) {
-            if (!actions) {
+        fiddle: function(payload, selector, mode) {
+            if (mode == 'replace') {
+                $(selector).replaceWith(payload);
+                var context = $(selector);
+                if (context.length) {
+                    context.parent().bdajax();
+                } else {
+                    $(document).bdajax();
+                }
+            } else if (mode == 'inner') {
+                $(selector).html(payload);
+                $(selector).bdajax();
+            }
+        },
+        
+        continuation: function(definitions) {
+            if (!definitions) {
                 return;
             }
             bdajax.spinner.hide();
-            var action, target;
-            for (var idx in actions) {
-                action = actions[idx];
-                if (action.type == 'action') {
-                    target = bdajax.parsetarget(action.target);
+            var definition, target;
+            for (var idx in definitions) {
+                definition = definitions[idx];
+                if (definition.type == 'action') {
+                    target = bdajax.parsetarget(definition.target);
                     bdajax.action({
                         url: target.url,
                         params: target.params,
-                        name: action.name,
-                        mode: action.mode,
-                        selector: action.selector
+                        name: definition.name,
+                        mode: definition.mode,
+                        selector: definition.selector
                     });
-                } else if (action.type == 'event') {
-                    bdajax.trigger(action.name, action.selector, action.target);
+                } else if (definition.type == 'event') {
+                    bdajax.trigger(definition.name,
+                                   definition.selector,
+                                   definition.target);
+                } else if (definition.type == 'message') {
+                    if (definition.flavor) {
+                        var flavors = ['message', 'info', 'warning', 'error'];
+                        if (flavors.indexOf(definition.flavor) == -1) {
+                            throw "Continuation definition.flavor unknown";
+                        }
+                        switch (definition.flavor) {
+                            case 'message':
+                                bdajax.message(definition.payload);
+                                break;
+                            case 'info':
+                                bdajax.info(definition.payload);
+                                break;
+                            case 'warning':
+                                bdajax.warning(definition.payload);
+                                break;
+                            case 'error':
+                                bdajax.error(definition.payload);
+                                break;
+                        }
+                    } else {
+                        if (!definition.selector) {
+                            throw "Continuation definition.selector expected";
+                        }
+                        $(definition.selector).html(definition.payload);
+                    }
                 }
             }
         },
@@ -220,6 +252,7 @@
         
         overlay: function(options) {
             var elem = $('#ajax-overlay');
+            elem.removeData('overlay');
             elem.overlay({
                 mask: {
                     color: '#fff',
@@ -249,6 +282,7 @@
         
         message: function(message) {
             var elem = $('#ajax-message');
+            elem.removeData('overlay');
             elem.overlay({
                 mask: {
                     color: '#fff',
@@ -261,6 +295,10 @@
                 onLoad: function() {
                     elem.find('button:first').focus();
                 },
+                onBeforeClose: function() {
+                    var overlay = this.getOverlay();
+                    $('.message', overlay).empty();
+                },
                 oneInstance: false,
                 closeOnClick: false,
                 top:'20%'
@@ -314,9 +352,33 @@
             elem.data('overlay').load();
         },
         
-        _event_handler: function(event) {
+        _dispatching_handler: function(event) {
             event.preventDefault();
             var elem = $(this);
+            var options = {
+                elem: elem,
+                event: event
+            };
+            if (elem.attr('ajax:confirm')) {
+                options.message = elem.attr('ajax:confirm');
+                bdajax.dialog(options, bdajax._do_dispatching);
+            } else {
+                bdajax._do_dispatching(options);
+            }
+        },
+        
+        _do_dispatching: function(options) {
+            var elem = options.elem;
+            var event = options.event;
+            if (elem.attr('ajax:action')) {
+                bdajax._handle_ajax_action(elem, event);
+            }
+            if (elem.attr('ajax:event')) {
+                bdajax._handle_ajax_event(elem);
+            }
+        },
+        
+        _handle_ajax_event: function(elem) {
             var target = elem.attr('ajax:target');
             var defs = bdajax._defs_to_array(elem.attr('ajax:event'));
             for (var i = 0; i < defs.length; i++) {
@@ -326,9 +388,7 @@
             }
         },
         
-        _action_handler: function(event) {
-            event.preventDefault();
-            var elem = $(this);
+        _handle_ajax_action: function(elem, event) {
             var target;
             if (event.ajaxtarget) {
                 target = event.ajaxtarget;
@@ -360,4 +420,4 @@
         }
     };
 
-})(jQuery);
+})(jQuery);
\ No newline at end of file
